/*********************************************************************/
/*                                                                   */
/*   Program Name :  timesettk                                       */
/*                                                                   */
/*      Version:  0.00  2001-08-16                                   */
/*                1.00  2002-10-04                                   */
/*                1.10  2003-06-17  add logout                       */
/*                1.11  2003-07-20  add env variable monitor         */
/*                1.12  2005-06-12  minor update                     */
/*                1.13  2005-09-28  force to process in the case     */
/*                                   missing 1PPS                    */
/*                2.00  2005-12-20  support VSSP32                   */
/*                2.10  2007-02-23  revised to use common library    */
/*                2.11  2008-02-19  options '--help' and '--version' */
/*                                  were added                       */
/*                                                                   */
/*      Copyright (c) 2002-2008 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*
/*
     Time Set Program  
        How to use:
          timesettk year month day hour min sec

        Example:  set time to 2001/09/20 15:10:15

              timesettk 2001 9 20 15 10 15
              
        Timing of hit return key
        
           [K5/VSSP]
              just after 15:10:15
              
           [K5/VSSP32]
              just after 1 sec earlier time (i.e. 15:10:14)
              
           Time will be displayed for 10 sec after success of time set
             

*/
#define TIMESETTK_VERSION "Ver. 2008-02-19"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory


int main(int argc, char **argv)
{
    char dev[] = DEV_FILE; 
    int fd_in,rv, sig;
    unsigned int time,day,sec;
    unsigned int secold, stat;
    int  yyyy, mm, dd;
    int ddd;
    int  h, m, s; 
    int  y3; // 7 bits year  (0-127)
    int counter;
    int i;
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char logdir[60];               // logfile directory
    char hostname[20];             // host name
    char cbuf[80];
    
    if(argc == 2 && strncmp(*(argv+1),"--help",6)==0 ){ // 2008-02-19
        printf("\n");
        printf("timesettk %s\n", TIMESETTK_VERSION);
#ifdef _VSSP32
        printf("  compiled for K5/VSSP32 unit\n");
#else
        printf("  compiled for K5/VSSP (IP-VLBI) board\n");
#endif
        printf("\n");
        printf("Usage: timesettk year month day hour min sec\n");
        printf("\n");
        printf("Example:  set time to 2001/09/20 15:10:15\n");
        printf("\n");
        printf("      timesettk 2001 9 20 15 10 15\n");
        printf("\n");
        printf("Timing of hit return key is as follows;\n");
        printf("\n");
        printf("   [K5/VSSP (IP-VLBI board)]\n");
        printf("      just after 15:10:15\n");
        printf("   [K5/VSSP32]\n");
        printf("      just after 1 sec earlier time (i.e. 15:10:14)\n");
        printf("\n");
        printf("   Time will be displayed for 10 sec after success of time set\n");
        printf("\n");
        exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
        printf("\n");
        printf("timesettk %s\n", TIMESETTK_VERSION);
#ifdef _VSSP32
        printf("  compiled for K5/VSSP32 unit\n");
#else
        printf("  compiled for K5/VSSP (IP-VLBI) board\n");
#endif
        printf("\n");
        exit(0);
    }

    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 

    if(argc != 7){
        printf("timesettk %s\n  Usage: timesettk year month day hour min sec\n", TIMESETTK_VERSION);
        exit(0);
    }

    /* parameters get */
    yyyy = atoi(*(argv + 1 ));
    mm   = atoi(*(argv + 2 ));
    dd   = atoi(*(argv + 3 ));
    h    = atoi(*(argv + 4 ));
    m    = atoi(*(argv + 5 ));
    s    = atoi(*(argv + 6 ));

    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);

    sprintf(cbuf,"timesettk(%s) %d %d %d %d %d %d",TIMESETTK_VERSION,yyyy,mm,dd,h,m,s);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    fd_in = open(dev, O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        sprintf(cbuf,"device open() error");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    //printf("Device open successd No.[%d]\n",fd_in);
    
    /* board reference signal check */
    printf("\n");
    if((sig=board_ref_check2(fd_in))==-1) {
        close(fd_in);
        goto end;
    }
    
    /* time set */
    //printf("%d,%d,%d,%d,%d,%d\n",yyyy,mm,dd,h,m,s);  // debug

    //day = y2k(yyyy, mm, dd);
    md2ddd(yyyy, &ddd, mm, dd);
    day = (unsigned int)ddd;
    sec = ((h * 60) + m) * 60 + s;
    time = TDS_MAKE_TIME(day,sec);

    printf("   Set time  => Total day : %5d    Seconds from 0h : %5d \n",day,sec);
#ifdef _VSSP32
    //year set
    y3=yyyy-2000;
    rv = ioctl(fd_in, TDSIO_SET_YEAR, &y3);  
    if (rv == -1) {
        perror("ioctl() year set failed ");
        sprintf(cbuf,"ioctl() year set failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
#endif

    //date and seconds set to sampler
    rv = ioctl(fd_in, TDSIO_SET_TIME, &time);  
    if (rv == -1) {
        perror("ioctl() time set failed ");
        sprintf(cbuf,"ioctl() time set failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        close(fd_in);
        return -1;
    }
    //printf("time set ok\n");

    //sync to 1PPS
    if(sig==0){
        rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
        if (rv == -1) {
            perror("ioctl() 1pps sync failed ");
            sprintf(cbuf,"ioctl() 1pps sync failed");
            if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
            close(fd_in);
            return -1;
        }
        printf("   1pps sync ok\n");
    }

    time = 0; 
    secold=90000;
    counter=0;
    close(fd_in);
	
    
    // log out
    if(sig==0){
       sprintf(cbuf,"timesettk (time set OK)");
    } else {
       sprintf(cbuf,"timesettk (time set OK but 1PPS was not supplied! (no 1PPS sync made)");
    }
    if(LogWrite2(logdir,hostname,1,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");

    // display time for given time seconds
    printf("\n");
    i=board_time_disp(10);

#ifdef _VSSP32
    printf("\n\n*** time setting to K5/VSSP32 successed ");
#else
    printf("\n\n*** time setting successed ");
#endif
    if(sig !=0) printf("(but no 1PPS signal)");
    printf(" ***\n\n");
end:
    return 0;
}

